<?php
/**
 * Default application wide controller parent class
 *
 * @category App
 * @package App_Controller
 * @copyright company
 */

abstract class App_Controller extends Zend_Controller_Action
{
    /**
     * Store the available commands
     *
     * @var array
     */
    private $_commands = array();
    
    /**
     * Overrides init() from App_Controller
     * 
     * @access public
     * @return void
     */
    public function init()
    {
        parent::init();
        
        $this->t = Zend_Registry::get('Zend_Translate');
    }
    
    /**
     * Add a new command to the chain
     *
     * @param object $cmd
     * @return void
     */
    protected function _addCommand($cmd){
        if(is_object($cmd) && !array_key_exists(get_class($cmd), $this->_commands)){
            $this->_commands[get_class($cmd)] = $cmd;
        }
    }
    
    /**
     * Run a command through the command chain
     *
     * @param string $name
     * @param mixed $args
     * @return void
     */
    protected function _runCommand($name, $args){
        foreach($this->_commands as $cmd){
            if($result = $cmd->onCommand($name, $args)){
                return $result;
            }
        }
    }
    
    /**
     * Convenience method to get the paginator
     *
     * @param mixed $array 
     * @return void
     */
    protected function _getPaginator($array){
        $paginator = Zend_Paginator::factory($array);
        $paginator->setCurrentPageNumber($this->_getPage());
        $paginator->setItemCountPerPage(App_DI_Container::get('ConfigObject')->paginator->items_per_page);
        
        return $paginator;
    }
}
