<?php
/**
 * Custom form element for rendering the widget that allows to specify multiple
 * child birth years
 *
 * To override the default text, use the jsOptions array to send additional information
 * to the js component.
 *
 * Ex:
 *
 *  $childrens = new App_Form_Element_Childrens('childrens');
 *  $childrens->setOptions(
 *      array(
 *          'required' => TRUE,
 *          'multiOptions' => range(range(date('Y'), date('Y', strtotime('-100 years')))),
 *          'filters' => array(
 *              'StringTrim',
 *              'StripTags',
 *          ),
 *          'validators' => array(
 *              array(
 *                  'NotEmpty',
 *                  TRUE,
 *                  array(
 *                      'messages' => array(
 *                          Zend_Validate_NotEmpty::IS_EMPTY => $this->t('This field is required')
 *                      )
 *                  )
 *              ),
 *          ),
 *          'jsOptions' => array(
 *              'rowTitle' => 'This text will appear before the select',
 *              'selectDefaultOptionValue' => 'Value for the default options',
 *              'selectDefaultOptionText' => 'This text will be the default option',
 *              'addMoreButtonTitle' => 'This text will be the caption of the button',
 *          ),
 *      )
 *  );
 *
 *
 * @category App
 * @package App_Form
 * @copyright Familia Facil S.L
 */

class App_Form_Element_Childrens extends Zend_Form_Element_Multi
{
    /**
     * View helper used to render this element
     * 
     * @var string
     * @access public
     */
    public $helper = 'formChildrens';
    
    /**
     * View helper used to render the errors of this element
     *
     * @var string
     * @access public
     */
    public $errorHelper = 'formChildrensErrors';
    
    /**
     * Holds the errors
     *
     * @var array
     */
    public $errors = array();
    
    /**
     * Error messages
     */
    const IS_EMPTY = 'This field is required';
    
    /**
     * Multiselect is an array of values by default
     *
     * @var bool
     * @access public
     */
    protected $_isArray = true;
    
    /**
     * Initialize the form getting the translator
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        $this->t = Zend_Registry::get('Zend_Translate');
    }
    
    /**
     * Overrides isValid() in Zend_Form_Element_Multi and
     * removes the validators added by Zend_Form_Element_Multi 
     * (InArray validator, etc.)
     * 
     * @param array $value 
     * @param array $context 
     * @access public
     * @return bool
     */
    public function isValid($value, $context = null)
    {
        $this->setValue($value);
        $this->errors = array();
        
        $value = array_filter($value, 'strlen');
        $isValid = TRUE;
        
        if($this->isRequired() && empty($value)){
            $this->errors['childrens'] = self::IS_EMPTY;
            $isValid = FALSE;
        }
        
        return $isValid;
    }
}

