<?php
/**
 * Custom form element for rendering the widget that allows to specify multiple
 * child birth years
 *
 *
 * @category App
 * @package App_View
 * @copyright Familia Facil S.L.
 */

class App_View_Helper_FormChildrens extends App_View_Helper_FormElement
{
    /**
     * Convenience method
     * call $this->formChildrens() in the view to access 
     * the helper
     *
     * @param string $name 
     * @param array $value 
     * @param array $attributes 
     * @param array $options
     * @access public
     * @return string
     */
    public function formChildrens($name, $value = null, $attributes = null, $options = null)
    {
        $name = substr($name, 0, -2);
        
        $jsOptions = 'null';
        
        if (isset($attributes['jsOptions'])) {
            $jsOptions = Zend_Json::encode($attributes['jsOptions']);
        }
        
        if(is_array($value)){
            $value = array_filter($value, 'strlen');
        }
        
        $js = sprintf('
            $(function() {
                $("label[for=%1$s]").childrenWidget(["%1$s", %2$s, %3$s, %4$s]);
            });',
            $name,
            $this->_getOptions($options),
            ($value) ? Zend_Json::encode($value) : 'null',
            $jsOptions
        );
        
        $this->view->headScript()->appendFile('/js/children_widget.js');
        $this->view->headScript()->appendScript($js);
    }

    /**
     * Converts the options into an array
     * 
     * @param array $options 
     * @access protected
     * @return string
     */
    protected function _getOptions(array $options = null, $encode = true)
    {
        $json = array();
        foreach ($options as $key => $value) {
            $json []= array(
                'value' => $key,
                'label' => $value,
            );
        }
        
        return $encode ? Zend_Json::encode($json) : $json;
    }
}

