(function($) {
    var commands = {
        errors: errors
    };
    
    var config = {
        rowTitle : 'Child born on:',
        selectDefaultOptionValue : '',
        selectDefaultOptionText : 'Select the birth year of the child',
        addMoreButtonTitle : 'Add child'
    };
    
    var latest = 0;
    
    /**
     * Function exposed to js. The entry point and the router of functions
     */
    $.fn.childrenWidget = function() {
        if (typeof arguments[0] === 'string') {
            //execute string comand
            var property = arguments[1];
            //remove the command name from the arguments
            var args = Array.prototype.slice.call(arguments);
            args.splice(0, 1);
            
            commands[arguments[0]].apply(this, args);
        } else {
            createWidget.apply(this, arguments);
        }
        return this;
    };
    
    /**
     * Function to render the error messages
     *
     * @param array arguments
     */
    function errors(arguments){
        $.each(arguments, function(field, error){
            var element = $('label[for=' + field + '] select').first();
            
            if(element != null){
                element.parent().addClass('error');
                element.parent().append('<strong class="error">' + error + '</strong>');
            }
        });
    }
    
    /**
     * Create the widget
     *
     * @params array arguments
     */
    function createWidget(arguments)
    {
        config.name = arguments[0];
        config.dropDownOptions = arguments[1];
        config.data = arguments[2];
        settings = arguments[3];
        
        if (settings) {
            $.extend(config, settings);
        }
        
        /**
         * Populates the table with existing data if data was given, if not add
         * the default field
         *
         * @param jQueryElement e  the parent element
         */
        function _initData(e)
        {
            if (config.data) {
                $.each(config.data, function(i) {
                    _appendRow(e, config.data[i]);
                });
            }else if (config.data == null){
                _appendRow(e);
            }
        }
        
        /**
         * Initializes the "add entry" button and appends it to the parent
         * element
         *
         * @param jQueryElement e  the parent element
         */
        function _initAddButton(e)
        {
            var p = $('<p class="addelement clearfix"></p>');
            var a = $('<a href="#" class="button black medium">' + config.addMoreButtonTitle + '</a>');
            
            a.bind('click', function(){
                _appendRow(e);
                return false;
            });
            
            $(e).prepend(p.append(a));
        }
        
        /**
         * Creates and appends an entry row to the table inside the parent
         * element
         *
         * @param jQueryElement e   the parent element
         * @param array data        data to be appended
         */
        function _appendRow(e, data)
        {
            ++latest;
            
            if (!data) {
                data = null;
            }
            
            var p = $('<p class="select children-age-selector"></p>');
            var label = $('<label for="childrens"></label>');
            var title = $('<span>' + config.rowTitle + '</span>');
            
            $(e).find('p.addelement').before(p.append(label.append(title, _createDropdown(data))));
            
            if($('.children-age-selector').length > 1){
                $('.children-age-selector').each(function(i, e){
                    if(i < $('.children-age-selector').length - 1){
                        $(e).addClass('glued');
                    }
                });
            }
        }
        
        /**
         * Returns a new dropdown widget
         *
         * @param string selectedOption  value of the option to be selected
         *                                in the dropdown or null
         * @return jQueryElement
         */
        function _createDropdown(selectedOption)
        {
            var select = $('<select/>');
            
            select.attr('name', config.name + '[]')
                  .attr('id', config.name + '_' + latest);
            
            var option = $('<option/>');
            option.attr('value', config.selectDefaultOptionValue).text(config.selectDefaultOptionText);
            select.append(option);
            
            $.each(config.dropDownOptions, function() {
                var option = $('<option/>');
                option.attr('value', this.label).text(this.label);
                
                if (this.label == selectedOption) {
                    option.attr('selected', 'selected');
                }
                
                select.append(option);
            });
            
            return select;
        }
        
        this.each(function() {
            _initAddButton(this);
            _initData(this);
        });
        
        return this;
    };
})(jQuery);